==> bibledit-0.9.tar.gz: bibledit-0.9/src/chapter.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "usfm.h"
#include "chapter.h"
#include "bible.h"
#include "usfmtools.h"


Chapter::Chapter (vector <ustring>& inputlines, Usfm * usfm)
{
  for (unsigned int i = 0; i < inputlines.size (); i++)
    lines.push_back (inputlines[i]);
  myusfm = usfm;
}


Chapter::~Chapter ()
{
}


void
Chapter::get_verse (unsigned int versenumber, vector <ustring>& outputlines)
{
  outputlines.clear ();
  unsigned int begin, end;
  get_verse_boundaries (versenumber, begin, end);
  for (unsigned int i = begin; i <= end; i++) {
    outputlines.push_back (lines[i]);
  }
  previously_presented_verse = versenumber;     // Used later to accept the verse again.
}


void 
Chapter::set_verse (vector<ustring>& inputlines)
{
  // Get the two boundaries in the vector of strings, where the
  // verse begins and where it ends.
  unsigned int begin, end;
  get_verse_boundaries (previously_presented_verse, begin, end);
  // Using an temporal vector of lines, start filling it with
  // 1. the set of lines before the current verse starts.
  // 2. the current edited verse.
  // 3. the lines following the current verse.
  vector<string> temporal_lines;
  for (unsigned int i = 0; i < lines.size(); i++) {
    if ((i < begin) || (i > end)) {
      temporal_lines.push_back(lines[i]);
    } else {
      i = end;
      for (unsigned int i2 = 0; i2 < inputlines.size(); i2++) {
        temporal_lines.push_back(inputlines[i2]);
      }
    }
  }
  // Copy the temporal lines back to the book.
  lines.clear();
  for (unsigned int i = 0; i < temporal_lines.size(); i++) {
    lines.push_back(temporal_lines[i]);
  }
}


void
Chapter::get_verse_boundaries (unsigned int verse_number, unsigned int &begin,
                               unsigned int &end)
{
  // Initialize the begin and end.
  begin = 0;
  end = 0;
  // Now go through all the text and start looking for the beginning and the end of the requested verse.
  unsigned int CurrentVerse = 0;
  bool look_for_first_marker = true;
  for (unsigned int i = 0; i < lines.size (); i++)
    {
      ustring s = lines[i];
      s = usfm_extract_marker (s);
      if (myusfm->is_verse_number (s))
        {
          CurrentVerse++;
        }
      if (CurrentVerse == verse_number)
        {
          end = i;
          if (look_for_first_marker)
            {
              begin = i;
              look_for_first_marker = false;
            }
        }
      // Speed up trick: break out when we have what we want.
      if (CurrentVerse > verse_number)
        break;
    }
  // The first verse also will include any headings. Care for that here.
  if (verse_number == 1)
    begin = 0;
}

==> bibledit-1.3.tar.gz: bibledit-1.3/src/chapter.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "usfm.h"
#include "chapter.h"
#include "bible.h"
#include "usfmtools.h"


Chapter::Chapter (vector <ustring>& inputlines)
{
  for (unsigned int i = 0; i < inputlines.size (); i++)
    lines.push_back (inputlines[i]);
}


Chapter::~Chapter ()
{
}


void
Chapter::get_verse (unsigned int versenumber, vector <ustring>& outputlines)
{
  outputlines.clear ();
  unsigned int begin, end;
  get_verse_boundaries (versenumber, begin, end);
  for (unsigned int i = begin; i <= end; i++) {
    outputlines.push_back (lines[i]);
  }
  previously_presented_verse = versenumber;     // Used later to accept the verse again.
}


void 
Chapter::set_verse (vector<ustring>& inputlines)
{
  // Get the two boundaries in the vector of strings, where the
  // verse begins and where it ends.
  unsigned int begin, end;
  get_verse_boundaries (previously_presented_verse, begin, end);
  // Using an temporal vector of lines, start filling it with
  // 1. the set of lines before the current verse starts.
  // 2. the current edited verse.
  // 3. the lines following the current verse.
  vector<ustring> temporal_lines;
  for (unsigned int i = 0; i < lines.size(); i++) {
    if ((i < begin) || (i > end)) {
      temporal_lines.push_back(lines[i]);
    } else {
      i = end;
      for (unsigned int i2 = 0; i2 < inputlines.size(); i2++) {
        temporal_lines.push_back(inputlines[i2]);
      }
    }
  }
  // Copy the temporal lines back to the book.
  lines.clear();
  for (unsigned int i = 0; i < temporal_lines.size(); i++) {
    lines.push_back(temporal_lines[i]);
  }
}


void
Chapter::get_verse_boundaries (unsigned int verse_number, unsigned int &begin,
                               unsigned int &end)
{
  // Initialize the begin and end.
  begin = 0;
  end = 0;
  // Now go through all the text and start looking for the beginning and the end of the requested verse.
  unsigned int CurrentVerse = 0;
  bool look_for_first_marker = true;
  for (unsigned int i = 0; i < lines.size (); i++)
    {
      ustring s = lines[i];
      s = usfm_extract_marker (s);
      if (usfm_is_verse (s))
        {
          CurrentVerse++;
        }
      if (CurrentVerse == verse_number)
        {
          end = i;
          if (look_for_first_marker)
            {
              begin = i;
              look_for_first_marker = false;
            }
        }
      // Speed up trick: break out when we have what we want.
      if (CurrentVerse > verse_number)
        break;
    }
  // The first verse also will include any headings. Care for that here.
  if (verse_number == 1)
    begin = 0;
}
